import telnetlib
import re


class MemcachedStats:

    _client = None
    _key_regex = re.compile(b'ITEM (.*) \[(.*); (.*)\]')
    _slab_regex = re.compile(r'STAT items:(.*):number')
    _stat_regex = re.compile(r"STAT (.*) (.*)\r")

    def __init__(self, host='localhost', port='11211'):
        self._host = host
        self._port = port

    @property
    def client(self):
        if self._client is None:
            self._client = telnetlib.Telnet(self._host, self._port)
        return self._client

    def command(self, cmd):
        ' Write a command to telnet and return the response '
        self.client.write(cmd)
        return self.client.read_until(b'END')

    def key_details(self, sort=True, limit=9999999):
        ' Return a list of tuples containing keys and details '
        cmd = 'stats cachedump %s %s'
        keys = []
        for id in self.slab_ids():
            command_str = cmd % (id, limit)
            for key in self._key_regex.findall(self.command(command_str.encode('ascii') + b'\n')):
                keys.append(key)
        if sort:
            return sorted(keys)
        else:
            return keys

    def keys(self, sort=True, limit=9999999):
        ' Return a list of keys in use '
        return [key[0].decode('ascii') for key in self.key_details(sort=sort, limit=limit)]

    def slab_ids(self):
        ' Return a list of slab ids in use '
        return self._slab_regex.findall(str(self.command(b'stats items \n')))

    def stats(self):
        ' Return a dict containing memcached stats '
        return dict(self._stat_regex.findall(self.command(b'stats \n')))
